{
  MathVect - Vector Math.
  Copyright  2001 Lord Trancos.
  <lordtrancos@softhome.net>
}

unit MathVect;

// --------------------------------------------------------------------

interface

uses DirectXGraphics;

type TXYZ = (vectX, vectY, vectZ);

const HALF_PI = PI/2;
      PI_X2   = PI*2;

function  vect(const _x, _y, _z: single): TD3DVector;
function  vectAddV(const _v1, _v2: TD3DVector): TD3DVector;
function  vectSubV(const _v1, _v2: TD3DVector): TD3DVector;
function  vectMulV(const _v1, _v2: TD3DVector): TD3DVector;
function  vectDivV(const _v1, _v2: TD3DVector): TD3DVector;
function  vectAddS(const _v: TD3DVector; const _s: single): TD3DVector;
function  vectSubS(const _v: TD3DVector; const _s: single): TD3DVector;
function  vectMulS(const _v: TD3DVector; const _s: single): TD3DVector;
function  vectDivS(const _v: TD3DVector; const _s: single): TD3DVector;
function  vectInv(const _v: TD3DVector): TD3DVector;
function  vectDot(const _v1, _v2: TD3DVector): single;
function  vectLen(const _v: TD3DVector): single;
function  vectNormalize(const _v: TD3DVector): TD3DVector;
procedure vectSetLen(var _v: TD3DVector; const _len: single);
function  vectCross(const _v1, _v2: TD3DVector): TD3DVector; // = wedge
function  vectLargestComponent(const _v: TD3DVector): TXYZ;

// --------------------------------------------------------------------

implementation

// --------------------------------------------------------------------

function vect(const _x, _y, _z: single): TD3DVector;

begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

// --------------------------------------------------------------------

function vectAddV(const _v1, _v2: TD3DVector): TD3DVector;

begin
  Result.x := _v1.x + _v2.x;
  Result.y := _v1.y + _v2.y;
  Result.z := _v1.z + _v2.z;
end;

// --------------------------------------------------------------------

function vectSubV(const _v1, _v2: TD3DVector): TD3DVector;

begin
  Result.x := _v1.x - _v2.x;
  Result.y := _v1.y - _v2.y;
  Result.z := _v1.z - _v2.z;
end;

// --------------------------------------------------------------------

function vectMulV(const _v1, _v2: TD3DVector): TD3DVector;

begin
  Result.x := _v1.x * _v2.x;
  Result.y := _v1.y * _v2.y;
  Result.z := _v1.z * _v2.z;
end;

// --------------------------------------------------------------------

function vectDivV(const _v1, _v2: TD3DVector): TD3DVector;

begin
  Result.x := _v1.x / _v2.x;
  Result.y := _v1.y / _v2.y;
  Result.z := _v1.z / _v2.z;
end;

// --------------------------------------------------------------------

function vectAddS(const _v: TD3DVector; const _s: single): TD3DVector;

begin
  Result.x := _v.x + _s;
  Result.y := _v.y + _s;
  Result.z := _v.z + _s;
end;

// --------------------------------------------------------------------

function vectSubS(const _v: TD3DVector; const _s: single): TD3DVector;

begin
  Result.x := _v.x - _s;
  Result.y := _v.y - _s;
  Result.z := _v.z - _s;
end;

// --------------------------------------------------------------------

function vectMulS(const _v: TD3DVector; const _s: single): TD3DVector;

begin
  Result.x := _v.x * _s;
  Result.y := _v.y * _s;
  Result.z := _v.z * _s;
end;

// --------------------------------------------------------------------

function vectDivS(const _v: TD3DVector; const _s: single): TD3DVector;

begin
  Result.x := _v.x / _s;
  Result.y := _v.y / _s;
  Result.z := _v.z / _s;
end;

// --------------------------------------------------------------------

function vectInv(const _v: TD3DVector): TD3DVector;

begin
  Result.x := - _v.x;
  Result.y := - _v.y;
  Result.z := - _v.z;
end;

// --------------------------------------------------------------------

function vectDot(const _v1, _v2: TD3DVector): single;

begin
  Result := _v1.x * _v2.x + _v1.y * _v2.y + _v1.z * _v2.z;
end;

// --------------------------------------------------------------------

function vectLen(const _v: TD3DVector): single;

begin
  Result := sqrt(vectDot(_v, _v));
end;

// --------------------------------------------------------------------

function vectNormalize(const _v: TD3DVector): TD3DVector;

var _l : single;

begin
  _l := vectLen(_v);
  Result := vectDivS(_v, _l);
end;

// --------------------------------------------------------------------

procedure vectSetLen(var _v: TD3DVector; const _len: single);

begin
  _v := vectMulS(_v, _len / vectLen(_v));
end;

// --------------------------------------------------------------------

function vectCross(const _v1, _v2: TD3DVector): TD3DVector;

begin
  Result.x := (_v1.y * _v2.z) - (_v1.z * _v2.y);
  Result.y := (_v1.z * _v2.x) - (_v1.x * _v2.z);
  Result.z := (_v1.x * _v2.y) - (_v1.y * _v2.x);  
end;

// --------------------------------------------------------------------

function  vectLargestComponent(const _v: TD3DVector): TXYZ;

var _abs : single;

begin
  _abs := Abs(_v.x);
  Result := vectX;
  if _abs < Abs(_v.y) then begin _abs := Abs(_v.y); Result := vectY; end;
  if _abs < Abs(_v.z) then Result := vectZ;
end;

// --------------------------------------------------------------------

end.
