unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, D3DX8,
  DX8GFX, DX8DI, MathVect, DX8PartSys, Samples;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    procedure InitParticleSystem;
    procedure SetMatrices;
    procedure GetUserInput;
    procedure hHalt(_errMsg: pChar);
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
  public
    { Public declarations }
  end;

const
  FULLSCR = false;

  MAXYROT =  Pi/4;
  MINYROT = -Pi/6;

  TITLE = 'Sample X10 - Particles - ';
  USAGE = 'This sample shows some particle systems using my 1st very simple and weird particle engine.'#10#10+
          'Hit [ENTER] to change the sample and move the mouse to rotate de camera.';

  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';

  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error loading textures.';
  ERR02 = 'Error creating particle system.';
  ERR03 = 'Error initializing DirectInput.'#10+
          'You must have installed DirectX 8 or better.';
  ERR04 = 'Error initializing mouse.';

var
  Form1: TForm1;

  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  FRAMECONTROL  : dWord = 0;
  LASTFRAMETIME : dWord = 0;

  // Camera.
  XRot : single;
  YRot : single;
  CTarget   : TD3DVector;
  CPosition : TD3DVector;

// -------------------------------------------------------------------

implementation

{$R *.DFM}

// -------------------------------------------------------------------

procedure TForm1.hHalt(_errMsg: pChar);

begin
  MessageBox(Handle, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure TForm1.GetUserInput;

var _x, _y, _z : longint;

begin
  if DIGetMouseState(_x, _y, _z, _z, _z, _z, _z, _z) then
    begin
      XRot := XRot - _x / 200;
      YRot := YRot - _y / 200;

      if YRot > MAXYROT then YRot := MAXYROT;
      if YRot < MINYROT then YRot := MINYROT;
      if XRot < -(2*Pi) then XRot := XRot+(2*Pi);
      if XRot > +(2*Pi) then XRot := XRot-(2*Pi);

      CTarget.x := CPosition.x + (cos(XRot) * cos(YRot));
      CTarget.z := CPosition.z + (sin(XRot) * cos(YRot));
      CTarget.y := CPosition.y + (sin(YRot));

      SetMatrices;
    end else DIMouseControl(true);
end;

// -------------------------------------------------------------------

procedure TForm1.SetMatrices;

var _v : TD3DVector;
    _matWorld : TD3DMatrix; // World
    _matView  : TD3DMatrix; // View
    _matProj  : TD3DMatrix; // Projection

begin
  // World.
  D3DXMatrixIdentity(_matWorld);
  D3DDEV8.SetTransform(D3DTS_WORLD, _matWorld);

  // View.
  _v := D3DXVector3(0.0, 1.0, 0.0);
  D3DXMatrixLookAtLH(_matView, CPOSITION, CTARGET, _v);
  D3DDEV8.SetTransform(D3DTS_VIEW, _matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(_matProj, PI/3,
                             ClientWidth/ClientHeight, 0.1, 100);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, _matProj);
end;

// -------------------------------------------------------------------

procedure RenderFrame;

begin
  // Clear the back buffer.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                gBackColor, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Render particle system
  if gParticleSystem.ok then
  begin
    psUpdateSystem(gParticleSystem, LASTFRAMETIME, gLockVB,
                   CPOSITION, CTARGET, gCurrAni);
    psRenderSystem(gParticleSystem);
  end;

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

var _time : dWord;

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  GetUserInput;
  RenderFrame;
  while GetTickCount - FRAMECONTROL < 1 do Sleep(0);
  LASTFRAMETIME := GetTickCount - FRAMECONTROL;
  _time := GetTickCount;
  FRAMECONTROL := _time;
end;

// ------------------------------------------------------------------

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);

// Next particle system sample

begin
  InitSample;
  If (Key = #13) or (Key = #32) then
  begin
    gCurrPSystem := (gCurrPSystem + 1) mod NUM_SAMPLES;
    Caption := TITLE + SAMPLE_NAME[gCurrPSystem];
    // Init particle system
    InitParticleSystem;
    // Set default camera pos
    XRot := Pi/2;
    YRot := 0;
  end;
end;

// ------------------------------------------------------------------

procedure TForm1.InitParticleSystem;

begin
  if not InitSample then begin hHalt(ERR02); exit; end;
end;

// ------------------------------------------------------------------

procedure TForm1.FormActivate(Sender: TObject);

begin
  if DXReady then exit;

  // Initialize DirectInput.
  if not DIInit then begin hHalt(ERR03); exit; end;
  if not DIInitMouse(Handle) then begin hHalt(ERR04); exit; end;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, FULLSCR, 640, 480, 16, 1, true, true) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, FULLSCR, 0, 0, 0, 0, false, true) then
        begin hHalt(ERR00); exit; end;
    end;

  // Initialize Camera
  XRot := Pi/2;
  YRot := 0;
  CTarget   := vect(0.0, 0.0, 1.0);
  CPosition := vect(0.0, 0.0, 0.0);

  // Set diffuse blending for alpha set in vertices.
  D3DDEV8.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
{
  // Enable alpha testing (skips pixels with less than a certain alpha.)
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_ALPHAREF, 8);
  D3DDEV8.SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
}
  // ColorOP and AlphaOp
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

  // Turn off D3D lighting.
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, 0);

  // Enable dithering
  D3DDEV8.SetRenderState(D3DRS_DITHERENABLE, 1);

  // Set texture filtering
  D3DDEV8.SetTextureStageState(0, D3DTSS_MAGFILTER, ord(D3DTEXF_LINEAR));
  D3DDEV8.SetTextureStageState(0, D3DTSS_MINFILTER, ord(D3DTEXF_LINEAR));

  // Load textures
  if not LoadTextures then begin hHalt(ERR01); exit; end;

  // Init particle system
  InitParticleSystem;

  // All right
  DXReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);

begin
  FRAMECONTROL := GetTickCount;
  Application.Title := TITLE;
  Caption := TITLE + SAMPLE_NAME[gCurrPSystem];
  ShowMessage(USAGE);

  // Init particle system
  psResetSystem(gParticleSystem);

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;
end;

// ------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

begin
  DXReady := false;
  FreeTextures; // Free textures
  psFreeSystem(gParticleSystem); // Free Particle System
  CloseDGfx; // Close DirectGraphics
  DIClose; // Close DirectInput
end;

// ------------------------------------------------------------------

end.
