// Sample Particle Systems

unit samples;

// -------------------------------------------------------------------

interface

uses
  Windows, DirectXGraphics, D3DX8, DX8Gfx, DX8PartSys, MathVect;

const
  // Number of sample particle systems
  NUM_SAMPLES = 6;

  // Number of textures
  NUM_TEXTURES = 5;

  // Texture file names
  TEX_FILENAME : array[0..NUM_TEXTURES-1] of string = (
                   'star.png',
                   'rain.png',
                   'dot.png',
                   'water.png',
                   'smoke.png'
                   );

  // Sample names
  SAMPLE_NAME : array[0..NUM_SAMPLES-1] of string = (
                  'Universe',
                  'Rain',
                  'Magic',
                  'Fire',
                  'Water',
                  'Smoke'
                  );

var
  gParticleSystem : TPSSystem; // The particle system
  gTexture : array[0..NUM_TEXTURES-1] of IDirect3DTexture8; // Textures
  gCurrPSystem : byte = 0; // Current Particle System Sample Index
  gCurrAni : TPSAniPartCB = NIL; // Current Animation Callback function
  gBackColor : dWord = $FF000000; // Background color
  gLockVB : boolean = false; // Lock Vertex Buffer ?

procedure FreeTextures;
function  LoadTextures: boolean;
function  InitSample: boolean;

// -------------------------------------------------------------------

implementation

// -------------------------------------------------------------------

procedure FreeTextures;

var _cnt : longint;

begin
  for _cnt := 0 to NUM_TEXTURES - 1 do gTexture[_cnt] := NIL;
end;

// -------------------------------------------------------------------

function LoadTextures: boolean;

var _cnt : longint;

begin
  Result := false;
  // Reset all textures
  FreeTextures;
  // Load textures
  for _cnt := 0 to NUM_TEXTURES - 1 do
  begin
    if failed(D3DXCreateTextureFromFileEx(D3DDEV8, pChar(TEX_FILENAME[_cnt]),
                                          D3DX_DEFAULT, D3DX_DEFAULT,
                                          1, 0, D3DFMT_A4R4G4B4,
                                          D3DPOOL_DEFAULT,
                                          D3DX_FILTER_LINEAR,
                                          D3DX_FILTER_LINEAR, 0,
                                          NIL, NIL, gTexture[_cnt])) then
    begin
      FreeTextures;
      exit;
    end;
  end;
  Result := true;
end;

// -------------------------------------------------------------------

procedure newParticle2(const _p: PPSParticle); stdcall;

// Magic

begin
  _p^.Vel0.y := -(Random * 5);
  _p^.Vel0.x := -2 + (random * 4);
  _p^.Vel0.z := -2 + (random * 4);
  _p^.Color0 := dWord($FF7F7F7F) or dword(($808080 and random($FFFFFF)));
end;

// -------------------------------------------------------------------

procedure newParticle3(const _p: PPSParticle); stdcall;

// Fire

begin
  _p^.Vel0.y := (Random * 2);
  _p^.Vel0.x := -1 + (random * 2);
  _p^.Vel0.z := -1 + (random * 2);
  _p^.Color0 := dWord($FF7F7F1F) or dword(($808000 and random($FFFFFF)));
end;

// -------------------------------------------------------------------

procedure newParticle4(const _p: PPSParticle); stdcall;

// Water

begin
  _p^.Vel0.y := 10 + (Random * 5);
  _p^.Vel0.x := -2 + (random * 4);
  _p^.Vel0.z := -2 + (random * 4);
  _p^.Color0 := dWord($FF1F7F7F) or dword(($8080 and random($FFFFFF)));
end;

// -------------------------------------------------------------------

procedure newParticle5(const _p: PPSParticle); stdcall;

// Smoke

var _b : byte;

begin
  _p^.Vel0.y := 1 + (Random * 2);
  _p^.Vel0.x := -1 + (random * 2);
  _p^.Vel0.z := -1 + (random * 2);
  _b := $70 + random($F);
  _p^.Color0 := dWord($FF) + _b shl 16 + _b shl 8 + _b;
end;

// -------------------------------------------------------------------

procedure aniParticle1(const _p: PPSParticle;
                       const _q: PPSQuad); stdcall;

// Rain 

begin
  _p^.Pos.y := _p^.Pos0.y - (_p^.Age / 200);
  if (_p^.Pos.y < -10) then psNewParticle(@gParticleSystem, _p, $FFFFFFFF, NIL);
end;

// -------------------------------------------------------------------

procedure aniParticle2(const _p: PPSParticle;
                       const _q: PPSQuad); stdcall;

// Magic

var _t : single;
    _c : longint;
    _alpha : longint;

begin
  for _c := 0 to 3 do
  begin
    _alpha := (255 * _p^.age) div 1000;
    if _alpha > 255 then _alpha := 255;
    _alpha := 255 - _alpha;
    _q^[_c].color := (_q^[_c].color and $FFFFFF) or dword((_alpha shl 24));
  end;
  _t := (_p^.Age / 1000);
  _p^.Pos.y := _p^.Pos0.y + _p^.Vel0.y * _t + (6 * _t * _t);
  _p^.Pos.x := _p^.Pos0.x + _p^.Vel0.x * _t;
  _p^.Pos.z := _p^.Pos0.z + _p^.Vel0.z * _t;
  if (_p^.Pos.y > 10) then psNewParticle(@gParticleSystem, _p, $FFFFFFFF, newParticle2);
end;

// -------------------------------------------------------------------

procedure aniParticle3(const _p: PPSParticle;
                       const _q: PPSQuad); stdcall;

// Fire

var _t : single;
    _c : longint;
    _alpha : longint;

begin
  for _c := 0 to 3 do
  begin
    _alpha := (255 * _p^.age) div 1000;
    if _alpha > 255 then _alpha := 255;
    _alpha := 255 - _alpha;
    _q^[_c].color := (_q^[_c].color and $FFFFFF) or dWord(_alpha shl 24);
  end;
  _t := (_p^.Age / 1000);
  _p^.Pos.y := _p^.Pos0.y + _p^.Vel0.y * _t + (6 * _t * _t);
  _p^.Pos.x := _p^.Pos0.x + _p^.Vel0.x * _t;
  _p^.Pos.z := _p^.Pos0.z + _p^.Vel0.z * _t;
  if (_p^.Pos.y > 10) then psNewParticle(@gParticleSystem, _p, $FFFFFFFF, newParticle3);
end;

// -------------------------------------------------------------------

procedure aniParticle4(const _p: PPSParticle;
                       const _q: PPSQuad); stdcall;

// Water

var _t : single;
    _c : longint;
    _alpha : longint;

begin
  for _c := 0 to 3 do
  begin
    _alpha := (255 * _p^.age) div 1000;
    if _alpha > 255 then _alpha := 255;
    _alpha := 255 - _alpha;
    _q^[_c].color := (_q^[_c].color and $FFFFFF) or dWord(_alpha shl 24);
  end;
  _t := (_p^.Age / 1000);
  _p^.Pos.y := _p^.Pos0.y + _p^.Vel0.y * _t - (12 * _t * _t);
  _p^.Pos.x := _p^.Pos0.x + _p^.Vel0.x * _t;
  _p^.Pos.z := _p^.Pos0.z + _p^.Vel0.z * _t;
  if (_p^.Pos.y < -5) then psNewParticle(@gParticleSystem, _p, $FFFFFFFF, newParticle4);
end;

// -------------------------------------------------------------------

procedure aniParticle5(const _p: PPSParticle;
                       const _q: PPSQuad); stdcall;

// Smoke

var _t : single;
    _c : longint;
    _alpha : longint;

begin
  for _c := 0 to 3 do
  begin
    // modify alpha
    _alpha := (255 * _p^.age) div 2000;
    if _alpha > 255 then _alpha := 255;
    _alpha := 255 - _alpha;
    _q^[_c].color := (_q^[_c].color and $FFFFFF) or dWord(_alpha shl 24);
  end;
  // Scale particle
  _p^.Scale.x := 1 + ((3.5 * _p^.age) / 2000);
  _p^.Scale.y := 1 + ((1.5 * _p^.age) / 2000);
  // Move
  _t := (_p^.Age / 1000);
  _p^.Pos.y := _p^.Pos0.y + _p^.Vel0.y * _t + 0.1 * (_t * _t);
  _p^.Pos.x := _p^.Pos0.x + _p^.Vel0.x * _t;
  _p^.Pos.z := _p^.Pos0.z + _p^.Vel0.z * _t;
  if (_p^.Pos.y > 5) then psNewParticle(@gParticleSystem, _p, $FFFFFFFF, newParticle5);
end;

// -------------------------------------------------------------------

function InitSample0: boolean;

// Universe

begin
  // No VB lock
  gLockVB := false;
  // Background color
  gBackColor := $FF000000;
  // No fog
  D3DDEV8.SetRenderState(D3DRS_FOGENABLE, 0);
  // Dest Blend
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  // No animation
  gCurrAni := NIL;
  // Set texture
  D3DDEV8.SetTexture(0, gTexture[0]);
  // Create particle system
  gParticleSystem := psNewSystem(psEmitter_AAB(vect( -20, -20, -20),
                                               vect(  20,  20,  20)),
                                 aaNone, 2000, $FFD0D0FF, 0.25, 0.25, NIL);
  Result := gParticleSystem.ok;
end;

// -------------------------------------------------------------------

function InitSample1: boolean;

// Rain

var _fogd : single;

begin
  // No VB lock
  gLockVB := false;
  // Background color
  gBackColor := $FF505050;
  // Fog
  _fogd := 0.3;
  D3DDEV8.SetRenderState(D3DRS_FOGENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_FOGCOLOR, gBackColor);
  D3DDEV8.SetRenderState(D3DRS_FOGTABLEMODE, D3DFOG_EXP);
  D3DDEV8.SetRenderState(D3DRS_FOGDENSITY, PDWord(@_fogd)^);
  // Dest Blend
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  // Rain Animation
  gCurrAni := aniParticle1;
  // Set texture
  D3DDEV8.SetTexture(0, gTexture[1]);
  // Create particle system
  gParticleSystem := psNewSystem(psEmitter_AAB(vect( -2,  5, -2),
                                               vect(  2, 10,  2)),
                                 aaY, 3000, $FFFFFFFF, 0.25, 0.25, NIL);
  Result := gParticleSystem.ok;
end;

// -------------------------------------------------------------------

function InitSample2: boolean;

// Magic

begin
  // VB lock
  gLockVB := true;
  // Background color
  gBackColor := $FF000050;
  // No Fog
  D3DDEV8.SetRenderState(D3DRS_FOGENABLE, 0);
  // Dest Blend
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
  // Animation
  gCurrAni := aniParticle2;
  // Set texture
  D3DDEV8.SetTexture(0, gTexture[2]);
  // Create particle system
  gParticleSystem := psNewSystem(psEmitter_Point(vect( 0, -2, 7)),
                                 aaNone, 150, $FFFFFFFF, 0.75, 0.75, newParticle2);
  Result := gParticleSystem.ok;
end;

// -------------------------------------------------------------------

function InitSample3: boolean;

// Fire

begin
  // VB lock
  gLockVB := true;
  // Background color
  gBackColor := $FF500000;
  // No Fog
  D3DDEV8.SetRenderState(D3DRS_FOGENABLE, 0);
  // Dest Blend
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
  // Animation
  gCurrAni := aniParticle3;
  // Set texture
  D3DDEV8.SetTexture(0, gTexture[2]);
  // Create particle system
  gParticleSystem := psNewSystem(psEmitter_Point(vect( 0, -3, 7)),
                                 aaNone, 100, $FFFFFFFF, 0.75, 1.5, newParticle3);
  Result := gParticleSystem.ok;
end;

// -------------------------------------------------------------------

function InitSample4: boolean;

// Water

begin
  // VB lock
  gLockVB := true;
  // Background color
  gBackColor := $FF005050;
  // No Fog
  D3DDEV8.SetRenderState(D3DRS_FOGENABLE, 0);
  // Dest Blend
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
  // Animation
  gCurrAni := aniParticle4;
  // Set texture
  D3DDEV8.SetTexture(0, gTexture[3]);
  // Create particle system
  gParticleSystem := psNewSystem(psEmitter_Point(vect( 0, -3, 4)),
                                 aaNone, 300, $FFFFFFFF, 0.2, 0.2, newParticle4);
  Result := gParticleSystem.ok;
end;

// -------------------------------------------------------------------

function InitSample5: boolean;

// Smoke

begin
  // VB lock
  gLockVB := true;
  // Background color
  gBackColor := $FF505050;
  // No Fog
  D3DDEV8.SetRenderState(D3DRS_FOGENABLE, 0);
  // Dest Blend
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  // Animation
  gCurrAni := aniParticle5;
  // Set texture
  D3DDEV8.SetTexture(0, gTexture[4]);
  // Create particle system
  gParticleSystem := psNewSystem(psEmitter_Point(vect( 0, -2, 4)),
                                 aaNone, 75, $FFFFFFFF, 0.5, 0.5, newParticle5);
  Result := gParticleSystem.ok;
end;

// -------------------------------------------------------------------

function InitSample: boolean;

begin
  // Free previous particle system
  psFreeSystem(gParticleSystem);
  // Create particle system
  Case gCurrPSystem of
    0 : Result := InitSample0;
    1 : Result := InitSample1;
    2 : Result := InitSample2;
    3 : Result := InitSample3;
    4 : Result := InitSample4;
    5 : Result := InitSample5;
    else Result := false;
  end;
  // Hidden animation,
  // comment this line if you want to know what it does
  psHiddenAnimation(gParticleSystem, 200, 200, gLockVB, gCurrAni);
end;

// -------------------------------------------------------------------

end.
