unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, DX8GFX, DX8DI, D3DX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    procedure SetMatrices;
    procedure GetUserInput;
    procedure InitParticles;
    procedure hHalt(_errMsg: pChar);
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
  public
  end;

const
  FULLSCR = false;

  NUM_PARTICLES = 2000;
  NUM_TEXTURES  = 2;

  TEX_FILENAME : array[0..NUM_TEXTURES-1] of string = ('particle.png',
                                                       'spark.png');

  PART_WIDTH  = 0.33;
  PART_HEIGHT = 0.33;

  MAXYROT =  Pi/3;
  MINYROT = -Pi/3;

  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1;

  TITLE = 'Sample X9 - Billboard';
  CAP00 = ' (Y-Axis aligned, Press [Enter] to change)';
  CAP01 = ' (No axis aligned, Press [Enter] to change)';

  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';

  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error initializing VertexBuffer.';
  ERR02 = 'Error locking VertexBuffer.';
  ERR03 = 'Error initializing DirectInput.'#10+
          'You must have installed DirectX 8 or better.';
  ERR04 = 'Error initializing mouse.';
  ERR05 = 'Error loading file: ';

type
  TCustomVertex = record
                    pos   : TD3DVector;
                    color : dWord;
                    u, v  : single;
                  end;

  PParticle = ^TParticle;
  TParticle = record
                quad   : array[0..3] of TCustomVertex; // Billboard Quad
                pos    : TD3DVector; // Origin
                offset : dWord; // Offset inside vertex buffer of quad's vertices
                texId  : cardinal; // Texture ID
                d      : single; // Used for sorting particles
              end;

var
  Form1: TForm1;

  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  FRAMECONTROL : dWord = 0;

  gDir : TD3DVector; // Eye direction (needed for sorting)
  gParticlesVB : IDirect3DVertexBuffer8 = NIL; // Vertex Buffer
  gParticle : array[0..NUM_PARTICLES-1] of TParticle; // Particles
  gBillboardMTX : TD3DMatrix; // Matrix used for drawing billboards
  gTexture : array[0..NUM_TEXTURES-1] of IDirect3DTexture8;
  gBillYAligned : boolean = false; // Billboard Y Axis Aligned ?

  // Camera.
  XRot : single;
  YRot : single;
  CTarget   : TD3DVector;
  CPosition : TD3DVector;

implementation

{$R *.DFM}

// ------------------------------------------------------------------

function D3DVector(_x, _y, _z: single): TD3DVector;

begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

// -------------------------------------------------------------------

procedure TForm1.hHalt(_errMsg: pChar);

begin
  MessageBox(Handle, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure TForm1.GetUserInput;

var _x, _y, _z : longint;

begin
  if DIGetMouseState(_x, _y, _z, _z, _z, _z, _z, _z) then
    begin
      XRot := XRot - _x / 200;
      YRot := YRot - _y / 200;

      if YRot > MAXYROT then YRot := MAXYROT;
      if YRot < MINYROT then YRot := MINYROT;
      if XRot < -(2*Pi) then XRot := XRot+(2*Pi);
      if XRot > +(2*Pi) then XRot := XRot-(2*Pi);

      CTarget.x := CPosition.x + (cos(XRot) * cos(YRot));
      CTarget.z := CPosition.z + (sin(XRot) * cos(YRot));
      CTarget.y := CPosition.y + (sin(YRot));

      SetMatrices;
    end else DIMouseControl(true);
end;

// -------------------------------------------------------------------

procedure TForm1.SetMatrices;

var _v : TD3DVector;
    _matWorld : TD3DMatrix; // World
    _matView  : TD3DMatrix; // View
    _matProj  : TD3DMatrix; // Projection

begin
  // World.
  D3DXMatrixIdentity(_matWorld);
  D3DDEV8.SetTransform(D3DTS_WORLD, _matWorld);

  // View.
  _v := D3DXVector3(0.0, 1.0, 0.0);
  D3DXMatrixLookAtLH(_matView, CPOSITION, CTARGET, _v);
  D3DDEV8.SetTransform(D3DTS_VIEW, _matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(_matProj, PI/3,
                             ClientWidth/ClientHeight, 0.1, 100);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, _matProj);
end;

// -------------------------------------------------------------------

procedure FrameMove;

  // ---------------------------------------

  function GetBillMatrix: TD3DMatrix;

  var _angle : single;
      _worldUp, _right, _up, _view : TD3DVector;

  begin
    _view    := gDir;
    _worldUp := D3DVector(0.0, 1.0, 0.0);
    _angle   := D3DXVec3Dot(_worldUp, _view);
    _up.x    := _worldUp.x - _angle*_view.x;
    _up.y    := _worldUp.y - _angle*_view.y;
    _up.z    := _worldUp.z - _angle*_view.z;
    D3DXVec3Normalize(_up, _up);
    D3DXVec3Cross(_right, _up, _view);

    with Result do
    begin
      _11 := _right.x; _21 := _up.x; _31 := _view.x;
      _12 := _right.y; _22 := _up.y; _32 := _view.y;
      _13 := _right.z; _23 := _up.z; _33 := _view.z;
      _14 := 0.0;
      _24 := 0.0;
      _34 := 0.0;
      _44 := 1.0;
    end;
  end;

  // ---------------------------------------

  function GetD(const _p: PParticle): single;

  begin
    if gBillYAligned
      then Result := _p^.pos.x * gDir.x + _p^.pos.z * gDir.z
        else Result := _p^.pos.x * gDir.x + _p^.pos.y * gDir.y + _p^.pos.z * gDir.z;
  end;

  // ---------------------------------------

  procedure QuickSort(_iLo, _iHi: longint);

  var _lo, _hi : longint;
      _mid : single;
      _p : TParticle;

  begin
    _lo := _iLo;
    _hi := _iHi;
    _mid := gParticle[(_lo + _hi) div 2].d;
    repeat
      while gParticle[_lo].d < _mid do inc(_lo);
      while gParticle[_hi].d > _mid do dec(_hi);
      if _lo <= _hi then
      begin
        // Swap
        _p := gParticle[_lo];
        gParticle[_lo] := gParticle[_hi];
        gParticle[_hi] := _p;
        inc(_lo);
        dec(_hi);
      end;
    until _lo > _hi;
    if _hi > _iLo then QuickSort(_iLo, _hi);
    if _lo < _iHi then QuickSort(_Lo, _iHi);
  end;

  // ---------------------------------------

var _i : cardinal;

begin
  // gDir
  gDir.x := CTARGET.x - CPOSITION.x;
  gDir.y := CTARGET.y - CPOSITION.y;
  gDir.z := CTARGET.z - CPOSITION.z;
  D3DXVec3Normalize(gDir, gDir);
  // gBillboardMTX
  if gBillYAligned then
  begin
    if (gDir.x > 0)
      then D3DXMatrixRotationY(gBillboardMTX, -arctan(gDir.z/gDir.x)+PI/2)
        else D3DXMatrixRotationY(gBillboardMTX, -arctan(gDir.z/gDir.x)-PI/2);
  end else gBillboardMTX := GetBillMatrix;
  // Fill data used to sort the particles
  for _i := 0 to NUM_PARTICLES-1 do gParticle[_i].d := GetD(@gParticle[_i]);
  // Sort
  QuickSort(0, NUM_PARTICLES-1);
end;

// -------------------------------------------------------------------

procedure RenderParticles;

var _cnt : longint;

begin
  // Loop through and render all particles
  D3DDEV8.SetStreamSource(0, gParticlesVB, sizeof(TCustomVertex));
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);
  for _cnt := NUM_PARTICLES-1 downto 0 do
  begin
    // Translate the billboard into place
    gBillboardMTX._41 := gParticle[_cnt].pos.x;
    gBillboardMTX._42 := gParticle[_cnt].pos.y;
    gBillboardMTX._43 := gParticle[_cnt].pos.z;
    D3DDEV8.SetTransform(D3DTS_WORLD, gBillboardMTX);
    // Set texture
    D3DDEV8.SetTexture(0, gTexture[gParticle[_cnt].texID]);
    // Render the billboard
    D3DDEV8.DrawPrimitive(D3DPT_TRIANGLESTRIP, gParticle[_cnt].offset, 2);
  end;
end;

// -------------------------------------------------------------------

procedure RenderFrame;

begin
  // Clear the back buffer.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                $FF000050, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Render Particles
  RenderParticles;

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

var _time : dWord;

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  GetUserInput;
  FrameMove;
  RenderFrame;
  while GetTickCount - FRAMECONTROL < 1 do Sleep(0);
  _time := GetTickCount;
  FRAMECONTROL := _time;
end;

// ------------------------------------------------------------------

procedure TForm1.InitParticles;

var _i : longint;
    _p : pByte;
    _color : dWord;

begin
  // *** Initialize gParticle
  for _i := 0 to NUM_PARTICLES - 1 do
  begin
    // Set a Random Position for each one
    gParticle[_i].pos.x := ((Random(30000)-15000) / 1000);
    gParticle[_i].pos.y := ((Random(30000)-15000) / 1000);
    gParticle[_i].pos.z := ((Random(30000)-15000) / 1000);
    // Random texture
    gParticle[_i].texID := Random(NUM_TEXTURES);
    // Random color
    _color := dWord($FF7F7F7F) or ($FF808080 and dWord(random($FFFFFF)));
    // Quad
    with gParticle[_i] do
    begin
      quad[0].pos   := D3DVector(-PART_WIDTH, -PART_HEIGHT, 0.0);
      quad[0].color := _color;
      quad[0].u     := 0.0;
      quad[0].v     := 1.0;
      quad[1].pos   := D3DVector(-PART_WIDTH, +PART_HEIGHT, 0.0);
      quad[1].color := _color;
      quad[1].u     := 0.0;
      quad[1].v     := 0.0;
      quad[2].pos   := D3DVector( PART_WIDTH, -PART_HEIGHT, 0.0);
      quad[2].color := _color;
      quad[2].u     := 1.0;
      quad[2].v     := 1.0;
      quad[3].pos   := D3DVector( PART_WIDTH, +PART_HEIGHT, 0.0);
      quad[3].color := _color;
      quad[3].u     := 1.0;
      quad[3].v     := 0.0;
    end;
  end;

  // *** Initialize gParticlesVB
  // Create
  if (failed(D3DDEV8.CreateVertexBuffer(NUM_PARTICLES*4*SizeOf(TCustomVertex),
                                        D3DUSAGE_WRITEONLY, D3DFVF_CUSTOMVERTEX,
                                        D3DPOOL_MANAGED, gParticlesVB))) then
  begin hHalt(ERR01); exit; end;

  // Lock
  if (failed(gParticlesVB.Lock(0, 0, _p, 0))) then
  begin hHalt(ERR02); exit; end;

  // Fill
  for _i := 0 to NUM_PARTICLES - 1 do
  begin
    Move(gParticle[_i].quad, _p^, SizeOf(TCustomVertex)*4);
    gParticle[_i].offset := _i*4;
    inc(longint(pointer(_p)), SizeOf(TCustomVertex)*4);
  end;

  // Unlock
  gParticlesVB.Unlock();
end;

// ------------------------------------------------------------------

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);

begin
  // Change between "Y axis aligned" and "not Y axis aligned".
  if Key = #13 then gBillYAligned := not gBillYAligned;
  if gBillYAligned
    then Caption := TITLE + CAP00
      else Caption := TITLE + CAP01;
end;

// ------------------------------------------------------------------

procedure TForm1.FormActivate(Sender: TObject);

var _i : longint;

begin
  if DXReady then exit;

  // Initialize DirectInput.
  if not DIInit then begin hHalt(ERR03); exit; end;
  if not DIInitMouse(Handle) then begin hHalt(ERR04); exit; end;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, FULLSCR, 640, 480, 16, 1, true, true) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, FULLSCR, 0, 0, 0, 0, false, true) then
        begin hHalt(ERR00); exit; end;
    end;

  // Initialize particles
  InitParticles;

  // Initialize textures
  for _i := 0 to NUM_TEXTURES-1 do
  begin
    gTexture[_i] := NIL;
    if failed(D3DXCreateTextureFromFileEx(D3DDEV8, pChar(TEX_FILENAME[_i]),
                                          D3DX_DEFAULT, D3DX_DEFAULT,
                                          1, 0, D3DFMT_A4R4G4B4,
                                          D3DPOOL_DEFAULT,
                                          D3DX_FILTER_LINEAR,
                                          D3DX_FILTER_LINEAR,
                                          $FF000000, NIL, NIL, gTexture[_i])) then
    begin
      hHalt(pChar(ERR05 + TEX_FILENAME[_i]));
      exit;
    end;
  end;

  // Initialize Camera
  XRot := Pi/2;
  YRot := 0;
  CTarget   := D3DVector(0.0, 0.0, 1.0);
  CPosition := D3DVector(0.0, 0.0, 0.0);

  // Set diffuse blending for alpha set in vertices.
  D3DDEV8.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
{
  // Enable alpha testing (skips pixels with less than a certain alpha.)
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_ALPHAREF, 8);
  D3DDEV8.SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
}
  // ColorOP and AlphaOp
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);

  // Turn off D3D lighting.
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, 0);

  // Enable dithering
  D3DDEV8.SetRenderState(D3DRS_DITHERENABLE, 1);

  // Set texture filtering
  D3DDEV8.SetTextureStageState(0, D3DTSS_MAGFILTER, ord(D3DTEXF_LINEAR));
  D3DDEV8.SetTextureStageState(0, D3DTSS_MINFILTER, ord(D3DTEXF_LINEAR));

  // All right
  DXReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);

begin
  Randomize;
  FRAMECONTROL := GetTickCount;
  Application.Title := TITLE;
  Caption := TITLE + CAP01;

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;
end;

// ------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

var _i : longint;

begin
  DXReady := false;
  for _i := 0 to NUM_TEXTURES-1 do gTexture[_i] := NIL;
  gParticlesVB := NIL; // Free VB
  CloseDGfx; // Close DirectGraphics
  DIClose; // Close DirectInput
end;

// ------------------------------------------------------------------

end.
